45#define PSY_3GPP_THR_SPREAD_HI 1.5f
46#define PSY_3GPP_THR_SPREAD_LOW 3.0f
48#define PSY_3GPP_EN_SPREAD_HI_L1 2.0f
50#define PSY_3GPP_EN_SPREAD_HI_L2 1.5f
52#define PSY_3GPP_EN_SPREAD_HI_S 1.5f
54#define PSY_3GPP_EN_SPREAD_LOW_L 3.0f
56#define PSY_3GPP_EN_SPREAD_LOW_S 2.0f
58#define PSY_3GPP_RPEMIN 0.01f
59#define PSY_3GPP_RPELEV 2.0f
61#define PSY_3GPP_C1 3.0f
62#define PSY_3GPP_C2 1.3219281f
63#define PSY_3GPP_C3 0.55935729f
65#define PSY_SNR_1DB 7.9432821e-1f
66#define PSY_SNR_25DB 3.1622776e-3f
68#define PSY_3GPP_SAVE_SLOPE_L -0.46666667f
69#define PSY_3GPP_SAVE_SLOPE_S -0.36363637f
70#define PSY_3GPP_SAVE_ADD_L -0.84285712f
71#define PSY_3GPP_SAVE_ADD_S -0.75f
72#define PSY_3GPP_SPEND_SLOPE_L 0.66666669f
73#define PSY_3GPP_SPEND_SLOPE_S 0.81818181f
74#define PSY_3GPP_SPEND_ADD_L -0.35f
75#define PSY_3GPP_SPEND_ADD_S -0.26111111f
76#define PSY_3GPP_CLIP_LO_L 0.2f
77#define PSY_3GPP_CLIP_LO_S 0.2f
78#define PSY_3GPP_CLIP_HI_L 0.95f
79#define PSY_3GPP_CLIP_HI_S 0.75f
81#define PSY_3GPP_AH_THR_LONG 0.5f
82#define PSY_3GPP_AH_THR_SHORT 0.63f
84#define PSY_PE_FORGET_SLOPE 511
92#define PSY_3GPP_BITS_TO_PE(bits) ((bits) * 1.18f)
93#define PSY_3GPP_PE_TO_BITS(bits) ((bits) / 1.18f)
96#define PSY_LAME_FIR_LEN 21
97#define AAC_BLOCK_SIZE_LONG 1024
98#define AAC_BLOCK_SIZE_SHORT 128
99#define AAC_NUM_BLOCKS_SHORT 8
100#define PSY_LAME_NUM_SUBBLOCKS 2
106#define PSY_LAME_PE_GAP 12
107#define PSY_LAME_PE_QUIET 0.4f
108#define PSY_LAME_PE_RED 0.45f
112#define PSY_LAME_HIST 32
113#define PSY_LAME_NOV_BACK 30
250 -8.65163e-18 * 2, -0.00851586 * 2, -6.74764e-18 * 2, 0.0209036 * 2,
251 -3.36639e-17 * 2, -0.0438162 * 2, -1.54175e-17 * 2, 0.0931738 * 2,
252 -5.52212e-17 * 2, -0.313819 * 2
261 int lower_range = 12, upper_range = 12;
269 for (
i = 1;
i < 13;
i++) {
312 return 13.3f *
atanf(0.00076f *
f) + 3.5f *
atanf((
f / 7500.0f) * (
f / 7500.0f));
323 return 3.64 * pow(
f, -0.8)
324 - 6.8 *
exp(-0.6 * (
f - 3.4) * (
f - 3.4))
325 + 6.0 *
exp(-0.15 * (
f - 8.7) * (
f - 8.7))
326 + (0.6 + 0.04 * add) * 0.001 *
f *
f *
f *
f;
333 float prev, minscale, minath, minsnr, pe_min;
337 const float num_bark =
calc_bark((
float)bandwidth);
343 if (!
ctx->model_priv_data)
345 pctx =
ctx->model_priv_data;
350 chan_bitrate = (int)(chan_bitrate / 120.0 * (
ctx->avctx->global_quality ?
ctx->avctx->global_quality : 120));
358 ctx->bitres.size -=
ctx->bitres.size % 8;
361 for (j = 0; j < 2; j++) {
363 const uint8_t *band_sizes =
ctx->bands[j];
364 float line_to_frequency =
ctx->avctx->sample_rate / (j ? 256.f : 2048.0f);
365 float avg_chan_bits = chan_bitrate * (j ? 128.0f : 1024.0f) /
ctx->avctx->sample_rate;
374 for (
g = 0;
g <
ctx->num_bands[j];
g++) {
377 coeffs[
g].
barks = (bark + prev) / 2.0;
380 for (
g = 0;
g <
ctx->num_bands[j] - 1;
g++) {
382 float bark_width = coeffs[
g+1].
barks - coeffs->
barks;
385 coeff->spread_low[1] =
ff_exp10(-bark_width * en_spread_low);
387 pe_min = bark_pe * bark_width;
388 minsnr =
exp2(pe_min / band_sizes[
g]) - 1.5f;
392 for (
g = 0;
g <
ctx->num_bands[j];
g++) {
393 minscale =
ath(start * line_to_frequency,
ATH_ADD);
394 for (
i = 1;
i < band_sizes[
g];
i++)
396 coeffs[
g].
ath = minscale - minath;
397 start += band_sizes[
g];
408 for (
i = 0;
i <
ctx->avctx->ch_layout.nb_channels;
i++)
423 ret = 0.7548f * (in -
state[0]) + 0.5095f *
state[1];
433 0xB6, 0x6C, 0xD8, 0xB2, 0x66, 0xC6, 0x96, 0x36, 0x36
441 const int16_t *audio,
447 int attack_ratio = br <= 16000 ? 18 : 10;
450 uint8_t grouping = 0;
456 int switch_to_eight = 0;
457 float sum = 0.0, sum2 = 0.0;
460 for (
i = 0;
i < 8;
i++) {
461 for (j = 0; j < 128; j++) {
468 for (
i = 0;
i < 8;
i++) {
503 for (
i = 0;
i < 3;
i++)
515 for (
i = 0;
i < 8;
i++) {
516 if (!((grouping >>
i) & 1))
535 float clipped_pe, bit_save, bit_spend, bit_factor, fill_level, forgetful_min_pe;
539 fill_level =
av_clipf((
float)
ctx->fill_level /
size, clip_low, clip_high);
541 bit_save = (fill_level + bitsave_add) * bitsave_slope;
542 assert(bit_save <= 0.3f && bit_save >= -0.05000001f);
543 bit_spend = (fill_level + bitspend_add) * bitspend_slope;
544 assert(bit_spend <= 0.5f && bit_spend >= -0.1f);
551 bit_factor = 1.0f - bit_save + ((bit_spend - bit_save) / (
ctx->pe.max -
ctx->pe.min)) * (clipped_pe -
ctx->pe.min);
559 ctx->pe.min =
FFMIN(pe, forgetful_min_pe);
565 ctx->frame_bits * bit_factor,
595 float thr_avg, reduction;
597 if(active_lines == 0.0)
600 thr_avg =
exp2f((
a - pe) / (4.0f * active_lines));
601 reduction =
exp2f((
a - desired_pe) / (4.0f * active_lines)) - thr_avg;
603 return FFMAX(reduction, 0.0f);
609 float thr = band->
thr;
613 thr =
sqrtf(thr) + reduction;
632 const uint8_t *band_sizes,
const float *coefs,
const int cutoff)
635 int start = 0, wstart = 0;
638 for (
g = 0;
g < num_bands;
g++) {
641 float form_factor = 0.0f;
644 if (wstart < cutoff) {
645 for (
i = 0;
i < band_sizes[
g];
i++) {
646 band->
energy += coefs[start+
i] * coefs[start+
i];
654 start += band_sizes[
g];
655 wstart += band_sizes[
g];
673 hpfsmpl[
i] = (sum1 + sum2) * 32768.0f;
686 float desired_bits, desired_pe, delta_pe, reduction=
NAN, spread_en[128] = {0};
687 float a = 0.0f, active_lines = 0.0f, norm_fac = 0.0f;
691 uint8_t s2l[16] = {0};
695 if (start_after_long) {
696 const uint8_t *ls =
ctx->bands[0];
697 const int ln =
ctx->num_bands[0];
698 const uint8_t *
ss =
ctx->bands[1];
699 int lacc = 0, sacc = 0, gl = 0;
700 for (
int gs = 0; gs < num_bands && gs < 16; gs++) {
701 int center8 = (sacc +
ss[gs] / 2) * 8;
702 while (gl < ln - 1 && lacc + ls[gl] <= center8) { lacc += ls[gl]; gl++; }
711 const int cutoff = bandwidth * 2048 / wi->
num_windows /
ctx->avctx->sample_rate;
714 calc_thr_3gpp(wi, num_bands, pch, band_sizes, coefs, cutoff);
721 spread_en[0] =
bands[0].energy;
722 for (
g = 1;
g < num_bands;
g++) {
724 spread_en[
w+
g] =
FFMAX(
bands[
g].energy, spread_en[
w+
g-1] * coeffs[
g].spread_hi[1]);
726 for (
g = num_bands - 2;
g >= 0;
g--) {
728 spread_en[
w+
g] =
FFMAX(spread_en[
w+
g], spread_en[
w+
g+1] * coeffs[
g].spread_low[1]);
731 for (
g = 0;
g < num_bands;
g++) {
739 else if (!
w && start_after_long)
752 if (spread_en[
w+
g] * avoid_hole_thr > band->
energy || coeffs[
g].
min_snr > 1.0f)
765 desired_pe = pe * (
ctx->avctx->global_quality ?
ctx->avctx->global_quality : 120) / (2 * 2.5f * 120.0f);
770 if (
ctx->bitres.bits > 0) {
785 if (
ctx->bitres.bits > 0)
790 ctx->bitres.alloc = desired_bits;
792 if (desired_pe < pe) {
799 for (
g = 0;
g < num_bands;
g++) {
811 for (
i = 0;
i < 2;
i++) {
812 float pe_no_ah = 0.0f, desired_pe_no_ah;
813 active_lines =
a = 0.0f;
815 for (
g = 0;
g < num_bands;
g++) {
819 pe_no_ah += band->
pe;
825 desired_pe_no_ah =
FFMAX(desired_pe - (pe - pe_no_ah), 0.0f);
826 if (active_lines > 0.0f)
831 for (
g = 0;
g < num_bands;
g++) {
834 if (active_lines > 0.0f)
837 if (band->
thr > 0.0f)
844 delta_pe = desired_pe - pe;
845 if (
fabs(delta_pe) > 0.05f * desired_pe)
849 if (pe < 1.15f * desired_pe) {
851 norm_fac = norm_fac ? 1.0f / norm_fac : 0;
853 for (
g = 0;
g < num_bands;
g++) {
857 float delta_sfb_pe = band->
norm_fac * norm_fac * delta_pe;
858 float thr = band->
thr;
870 while (pe > desired_pe &&
g--) {
885 for (
g = 0;
g < num_bands;
g++) {
924 for (ch = 0; ch < group->
num_ch; ch++) {
959 ctx->next_window_seq = blocktype;
967 const float *la,
int channel,
int prev_type,
970 int uselongblock = 1;
975 const float *pf = hpfsmpl;
990 energy_short[0] += energy_subshort[
i];
996 for (; pf < pfe; pf++)
1014 float frame_peak = 1.0f;
1016 const float nov_gate = 1.25f;
1021 frame_peak =
FFMAX(frame_peak, energy_subshort[
i]);
1029 if (attack_intensity[
i] > thr) {
1037 float nearmax = 1.0f, deepmax = 1.0f;
1038 for (
int k = 3; k <= 8; k++)
1039 nearmax =
FFMAX(nearmax, env[
pos - k]);
1041 deepmax =
FFMAX(deepmax, env[
pos - k]);
1042 if (energy_subshort[
i] < nov_gate * nearmax ||
1043 (energy_subshort[
i] < nov_gate * deepmax &&
1057 const float u = energy_short[
i - 1];
1058 const float v = energy_short[
i];
1059 const float m =
FFMAX(
u, v);
1061 if (
u < 2.3f * v && v < 2.3f *
u) {
1062 if (
i == 1 && attacks[0] < attacks[
i])
1067 att_sum += attacks[
i];
1086 att_sum += attacks[0];
1094 if (attacks[
i] && attacks[
i-1])
1102 return uselongblock;
1110 int prev_type,
int have_la)
1136 for (
i = 0;
i < 8;
i++) {
1149 for (
i = 0;
i < 9;
i++) {
1163 const float *la,
int channel,
int prev_type)
1170 return psy_lame_apply(pctx, pch, uselongblock, attacks, prev_type, !!la);
1175 const float *audio0,
const float *la0,
1176 const float *audio1,
const float *la1,
1177 int channel0,
int channel1,
1178 int prev_type0,
int prev_type1,
1187 int u0 =
psy_lame_detect(pctx, pch0, la0, channel0, prev_type0, att0);
1188 int u1 =
psy_lame_detect(pctx, pch1, la1, channel1, prev_type1, att1);
1191 if (
ctx->pair_decoupled[(channel0 >> 1) & 15]) {
1205 merged[
i] = att0[
i] ? att0[
i] : att1[
i];
1213 .name =
"3GPP TS 26.403-inspired model",
AAC definitions and structures.
static float lame_calc_attack_threshold(int bitrate)
Calculate the ABR attack threshold from the above LAME psymodel table.
static av_cold void lame_window_init(AacPsyContext *ctx, AVCodecContext *avctx)
LAME psy model specific initialization.
static av_unused FFPsyWindowInfo psy_3gpp_window(FFPsyContext *ctx, const int16_t *audio, const int16_t *la, int channel, int prev_type)
Tell encoder which window types to use.
#define PSY_LAME_PE_RED
attack-threshold multiplier for a qualifying isolated onset
#define PSY_3GPP_SPEND_SLOPE_L
static void lame_apply_block_type(AacPsyChannel *ctx, FFPsyWindowInfo *wi, int uselongblock)
#define PSY_3GPP_CLIP_HI_L
#define PSY_3GPP_EN_SPREAD_HI_L1
#define PSY_LAME_PE_QUIET
pre-onset must be below this fraction of the frame peak
static const PsyLamePreset psy_abr_map[]
LAME psy model preset table for ABR.
#define PSY_3GPP_CLIP_LO_S
#define PSY_3GPP_AH_THR_LONG
static const float psy_fir_coeffs[]
LAME psy model FIR coefficient table.
static av_cold float calc_bark(float f)
Calculate Bark value for given line.
static void psy_lame_window_pair(FFPsyContext *ctx, const float *audio0, const float *la0, const float *audio1, const float *la1, int channel0, int channel1, int prev_type0, int prev_type1, FFPsyWindowInfo wi[2])
static float calc_reduced_thr_3gpp(AacPsyBand *band, float min_snr, float reduction)
#define PSY_3GPP_PE_TO_BITS(bits)
#define PSY_3GPP_THR_SPREAD_HI
constants for 3GPP AAC psychoacoustic model
static int psy_lame_detect(AacPsyContext *pctx, AacPsyChannel *pch, const float *la, int channel, int prev_type, int attacks[AAC_NUM_BLOCKS_SHORT+1])
static void psy_3gpp_analyze(FFPsyContext *ctx, int channel, const float **coeffs, const FFPsyWindowInfo *wi)
static float calc_reduction_3gpp(float a, float desired_pe, float pe, float active_lines)
static FFPsyWindowInfo psy_lame_window(FFPsyContext *ctx, const float *audio, const float *la, int channel, int prev_type)
#define PSY_3GPP_EN_SPREAD_HI_S
static void psy_hp_filter(const float *firbuf, float *hpfsmpl, const float *psy_fir_coeffs)
static const uint8_t window_grouping[9]
window grouping information stored as bits (0 - new group, 1 - group continues)
#define PSY_3GPP_THR_SPREAD_LOW
#define PSY_3GPP_EN_SPREAD_LOW_L
#define PSY_3GPP_SAVE_SLOPE_L
#define PSY_PE_FORGET_SLOPE
#define AAC_BLOCK_SIZE_LONG
long block size
#define PSY_3GPP_CLIP_LO_L
#define PSY_3GPP_BITS_TO_PE(bits)
#define PSY_3GPP_SAVE_ADD_L
#define PSY_LAME_HIST
HP sub-block peak history depth.
#define AAC_BLOCK_SIZE_SHORT
short block size
static float calc_pe_3gpp(AacPsyBand *band)
#define PSY_3GPP_AH_THR_SHORT
#define PSY_LAME_PE_GAP
min consecutive long frames before the relaxation applies
static const PsyLamePreset psy_vbr_map[]
LAME psy model preset table for constant quality.
#define PSY_LAME_NUM_SUBBLOCKS
Number of sub-blocks in each short block.
static av_cold float ath(float f, float add)
Calculate ATH value for given frequency.
#define AAC_NUM_BLOCKS_SHORT
number of blocks in a short sequence
static void psy_3gpp_analyze_channel(FFPsyContext *ctx, int channel, const float *coefs, const FFPsyWindowInfo *wi)
Calculate band thresholds as suggested in 3GPP TS26.403.
#define PSY_3GPP_SPEND_SLOPE_S
#define PSY_3GPP_EN_SPREAD_LOW_S
static int calc_bit_demand(AacPsyContext *ctx, float pe, int bits, int size, int short_window)
#define PSY_3GPP_CLIP_HI_S
static FFPsyWindowInfo psy_lame_apply(AacPsyContext *pctx, AacPsyChannel *pch, int uselongblock, const int attacks[AAC_NUM_BLOCKS_SHORT+1], int prev_type, int have_la)
#define PSY_LAME_NOV_BACK
novelty look-back in sub-blocks
#define PSY_3GPP_SPEND_ADD_L
static av_cold int psy_3gpp_init(FFPsyContext *ctx)
static void calc_thr_3gpp(const FFPsyWindowInfo *wi, const int num_bands, AacPsyChannel *pch, const uint8_t *band_sizes, const float *coefs, const int cutoff)
#define PSY_3GPP_SAVE_SLOPE_S
#define PSY_LAME_FIR_LEN
LAME psy model FIR order.
const FFPsyModel ff_aac_psy_model
#define PSY_3GPP_SPEND_ADD_S
#define PSY_3GPP_SAVE_ADD_S
static av_cold void psy_3gpp_end(FFPsyContext *apc)
static const float bands[]
Libavcodec external API header.
#define i(width, name, range_min, range_max)
#define ss(width, name, subs,...)
static __device__ float sqrtf(float a)
static __device__ float fabsf(float a)
static __device__ float fabs(float a)
channel
Use these values when setting the channel map with ebur128_set_channel().
static struct @346255127015250356166251341105367306144006377143 state
static const uint8_t bits[8]
internal math functions header
static av_always_inline double ff_exp10(double x)
Compute 10^x for floating point values.
#define iir_filter(fir_coef, iir_coef, src, dest, width)
Perform IIR filtering.
#define AV_CODEC_FLAG_QSCALE
Use fixed qscale.
#define FF_QP2LAMBDA
factor to convert from H.263 QP to lambda
#define u(width, name, range_min, range_max)
Macro definitions for various function/variable attributes.
void * av_calloc(size_t nmemb, size_t size)
Memory handling functions.
FFPsyChannelGroup * ff_psy_find_group(FFPsyContext *ctx, int channel)
Determine what group a channel belongs to.
int nb_channels
Number of channels in this layout.
main external API structure.
AVChannelLayout ch_layout
Audio channel layout.
int global_quality
Global quality for codecs which cannot change it per frame.
int64_t bit_rate
the average bitrate
int flags
AV_CODEC_FLAG_*.
information for single band used by 3GPP TS26.403-inspired psychoacoustic model
float thr_quiet
threshold in quiet
float norm_fac
normalization factor for linearization
float nz_lines
number of non-zero spectral lines
float active_lines
number of active spectral lines
float thr
energy threshold
int avoid_holes
hole avoidance flag
float pe
perceptual entropy
float pe_const
constant part of the PE calculation
single/pair channel context for psychoacoustic model
float attack_threshold
attack threshold for this channel
float iir_state[2]
hi-pass IIR filter state
float win_energy
sliding average of channel energy
int64_t win_count
window() calls so far (frame counter for pair sync)
int64_t rc_frame_num
frame this channel last saved rewind state for
int next_attack0_zero
whether attack[0] of the next frame is zero
int64_t last_att
win_count value of this channel's last own attack
enum WindowSequence next_window_seq
window sequence to be used in the next frame
float prev_energy_subshort[AAC_NUM_BLOCKS_SHORT *PSY_LAME_NUM_SUBBLOCKS]
AacPsyBand rc_prev_band[128]
prev_band as it was entering the frame
float hp_env_hist[PSY_LAME_HIST]
rolling HP sub-block peak envelope
AacPsyBand prev_band[128]
bands information from the previous frame
int frames_since_short
consecutive long frames (pre-echo-aware isolated-onset gate)
uint8_t next_grouping
stored grouping scheme for the next frame (in case of 8 short window sequence)
int prev_attack
attack value for the last short block in the previous sequence
float prev_frame_energy
previous frame's full-band lookahead energy (attack veto)
AacPsyBand band[128]
bands information
psychoacoustic model frame type-dependent coefficients
float ath
absolute threshold of hearing per bands
float spread_low[2]
spreading factor for low-to-high threshold spreading in long frame
float barks
Bark value for each spectral band in long frame.
float spread_hi[2]
spreading factor for high-to-low threshold spreading in long frame
3GPP TS26.403-inspired psychoacoustic model specific data
float correction
PE correction factor.
int rc_first_ch
first channel analyzed in that frame
int chan_bitrate
bitrate per channel
struct AacPsyContext::@245330130066201162145177070256170070220205206266 pe
float min
minimum allowed PE for bit factor calculation
AacPsyCoeffs psy_coef[2][64]
float global_quality
normalized global quality taken from avctx
int64_t rc_frame_num
frame the rewind state was saved for
float previous
allowed PE of the previous frame
int fill_level
bit reservoir fill level
float max
maximum allowed PE for bit factor calculation
int frame_bits
average bits per frame
single band psychoacoustic information
psychoacoustic information for an arbitrary group of channels
uint8_t num_ch
number of channels in this group
context used by psychoacoustic model
void * model_priv_data
psychoacoustic model implementation private data
codec-specific psychoacoustic model implementation
windowing related information
int num_windows
number of windows in a frame
int grouping[8]
window grouping (for e.g. AAC)
int window_shape
window shape (sine/KBD/whatever)
int window_type[3]
window type (short/long/transitional, etc.) - current, previous and next
LAME psy model preset struct.
float st_lrm
short threshold for L, R, and M channels
int quality
Quality to map the rest of the values to.
static AVFormatContext * ctx
static const double coeff[2][5]
static const uint8_t quality[]